JOB_TYPE_WORKER_CONTROL = "worker_control"
WORKER_CONTROL_ACTION_RETIRE = "retire"

# TUBE_DEFAULT_NAME = "default"
# BROKER_HOST_DEFAULT_ADDRESS = "fwq-the_broker"
BROKER_HOST_DEFAULT_PORT = 11300

JOB_START_PERSISTENCE_DELAY_IN_SECS = 1

JOB_STATE_FINAL = "finalized"
JOB_STATE_DONE = "done"
JOB_STATE_NEW = "new"
JOB_STATE_DELAYED = "delayed"
JOB_STATE_RESERVED = "reserved"
JOB_STATE_READY = "ready"
JOB_STATE_BURIED = "buried"

FINAL_JOB_STATES = [JOB_STATE_FINAL, JOB_STATE_BURIED]
PEEK_NEXT_JOB_STATES = [JOB_STATE_READY, JOB_STATE_DELAYED, JOB_STATE_BURIED]

STATS_TYPE_SYSTEM = "system"
STATS_TYPE_TUBE = "tube"
STATS_TYPE_JOB = "job"

BROKER_CONFIG_FROM_JOB_DATA = '_fwq_broker_config'
WORK_Q_CONFIG_FROM_JOB_DATA = '_fwq_work_q_config'
WORKER_CONFIG_FROM_JOB_DATA = '_fwq_worker_config'

